const path = require('path')
const cwd = process.cwd()

module.exports = () => ({
  visitor: {
    CallExpression: {
      enter (nodePath, { opts, file }) {
        const callee = nodePath.get('callee')
        if (callee.isIdentifier() && callee.equals('name', 'require')) {
          const arg = nodePath.get('arguments')[0]
          if (arg && arg.isStringLiteral()) {
            const result = resolve(arg.node.value, opts, file)
            if (result) arg.node.value = result
          }
        }
      }
    },
    ImportDeclaration: {
      enter (nodePath, { opts, file }) {
        const result = resolve(nodePath.node.source.value, opts, file)
        if (result) nodePath.node.source.value = result
      }
    }
  }
})

function resolve (name, opts, file) {
  if (!opts.exts) return
  const root = path.resolve(opts.root || cwd)
  const replace = path.join(root, opts.test || '')
  if (!path.isAbsolute(name) && opts.exts.includes(path.extname(name))) {
    return path
      .join(file.opts.sourceRoot, name)
      .replace(replace, root)
  }
}
